@@ -123,6 +123,15 @@ class BuildTest(unittest.TestCase):
         from speechd.client import SSIPClient
         del SSIPClient
 
+    @unittest.skipIf('SKIP_SPEECH_TESTS' in os.environ, 'Speech support is opted out')
+    def test_piper(self):
+        import subprocess
+
+        from calibre.constants import piper_cmdline
+        self.assertTrue(piper_cmdline())
+        raw = subprocess.check_output(piper_cmdline() + ('-h',), stderr=subprocess.STDOUT).decode()
+        self.assertIn('--sentence_silence', raw)
+
     def test_zeroconf(self):
         import ifaddr
         import zeroconf as z
@@ -299,6 +308,11 @@ class BuildTest(unittest.TestCase):
         m.close()
         self.assertEqual(winutil.parse_cmdline('"c:\\test exe.exe" "some arg" 2'), ('c:\\test exe.exe', 'some arg', '2'))
 
+    def test_ffmpeg(self):
+        from calibre_extensions.ffmpeg import resample_raw_audio_16bit
+        data = os.urandom(22050 * 2)
+        resample_raw_audio_16bit(data, 22050, 44100)
+
     def test_sqlite(self):
         import sqlite3
         conn = sqlite3.connect(':memory:')
@@ -314,7 +328,7 @@ class BuildTest(unittest.TestCase):
     def test_qt(self):
         if is_sanitized:
             raise unittest.SkipTest('Skipping Qt build test as sanitizer is enabled')
-        from qt.core import QApplication, QFontDatabase, QImageReader, QLoggingCategory, QNetworkAccessManager, QSslSocket, QTextToSpeech, QTimer
+        from qt.core import QApplication, QFontDatabase, QImageReader, QLoggingCategory, QNetworkAccessManager, QSslSocket, QTimer
         QLoggingCategory.setFilterRules('''qt.webenginecontext.debug=true''')
         if hasattr(os, 'geteuid') and os.geteuid() == 0:
             # likely a container build, webengine cannot run as root with sandbox
@@ -348,10 +362,15 @@ class BuildTest(unittest.TestCase):
         try:
             ensure_app()
             self.assertGreaterEqual(len(QFontDatabase.families()), 5, 'The QPA headless plugin is not able to locate enough system fonts via fontconfig')
-            available_tts_engines = tuple(x for x in QTextToSpeech.availableEngines() if x != 'mock')
-            self.assertTrue(available_tts_engines)
 
-            self.assertGreaterEqual
+            if 'SKIP_SPEECH_TESTS' not in os.environ:
+                from qt.core import QMediaDevices, QTextToSpeech
+
+                available_tts_engines = tuple(x for x in QTextToSpeech.availableEngines() if x != 'mock')
+                self.assertTrue(available_tts_engines)
+
+                QMediaDevices.audioOutputs()
+
             from calibre.ebooks.oeb.transforms.rasterize import rasterize_svg
             img = rasterize_svg(as_qimage=True)
             self.assertFalse(img.isNull())
@@ -401,6 +420,10 @@ class BuildTest(unittest.TestCase):
             if display_env_var is not None:
                 os.environ['DISPLAY'] = display_env_var
 
+    def test_pykakasi(self):
+        from calibre.ebooks.unihandecode.jadecoder import Jadecoder
+        self.assertEqual(Jadecoder().decode("自転車生活の愉しみ"), 'Jitensha Seikatsu no Tanoshi mi')
+
     def test_imaging(self):
         from PIL import Image
         try:
@@ -417,16 +440,14 @@ class BuildTest(unittest.TestCase):
         out = StringIO()
         features.pilinfo(out=out, supported_formats=False)
         out = out.getvalue()
-        for line in '''\
+        lines = '''\
         --- PIL CORE support ok
         --- FREETYPE2 support ok
         --- WEBP support ok
-        --- WEBP Transparency support ok
-        --- WEBPMUX support ok
-        --- WEBP Animation support ok
         --- JPEG support ok
         --- ZLIB (PNG/ZIP) support ok
-        '''.splitlines():
+        '''.splitlines()
+        for line in lines:
             self.assertIn(line.strip(), out)
         with Image.open(I('lt.png', allow_user_override=False)) as i:
             self.assertGreaterEqual(i.size, (20, 20))
@@ -553,8 +574,10 @@ def test_multiprocessing():
         p.join()
 
 
-def find_tests():
+def find_tests(only_build=False):
     ans = unittest.defaultTestLoader.loadTestsFromTestCase(BuildTest)
+    if only_build:
+        return ans
     from calibre.utils.icu_test import find_tests
     ans.addTests(find_tests())
     from tinycss.tests.main import find_tests
